@@ -15,6 +15,7 @@
 """Test the collection module."""
 import unittest
 
+import qcheck
 from test_connection import get_connection
 from pymongo.collection import Collection
 from pymongo.errors import InvalidName
@@ -127,6 +128,42 @@ class TestCollection(unittest.TestCase):
         options = {"capped": True}
         db.create_collection("test", options)
         self.assertEqual(db.test.options(), options)
+        db.drop_collection("test")
+
+    def test_insert_find_one(self):
+        db = self.db
+        db.test.remove({})
+        self.assertEqual(db.test.find().count(), 0)
+        doc = {"hello": u"world"}
+        db.test.insert(doc)
+        self.assertEqual(db.test.find().count(), 1)
+        self.assertEqual(doc, db.test.find_one())
+
+        def remove_insert_find_one(dict):
+            db.test.remove({})
+            db.test.insert(dict)
+            return db.test.find_one() == dict
+
+        qcheck.check_unittest(self, remove_insert_find_one, qcheck.gen_mongo_dict(3))
+
+    def test_iteration(self):
+        db = self.db
+
+        def iterate():
+            [a for a in db.test]
+
+        self.assertRaises(TypeError, iterate)
+
+    def test_insert_multiple(self):
+        db = self.db
+        db.drop_collection("test")
+        doc1 = {"hello": u"world"}
+        doc2 = {"hello": u"mike"}
+        self.assertEqual(db.test.find().count(), 0)
+        db.test.insert([doc1, doc2])
+        self.assertEqual(db.test.find().count(), 2)
+        self.assertEqual(doc1, db.test.find_one({"hello": u"world"}))
+        self.assertEqual(doc2, db.test.find_one({"hello": u"mike"}))
 
 if __name__ == "__main__":
     unittest.main()
