@@ -155,6 +155,70 @@ class TestDatabase(unittest.TestCase):
         self.assertTrue(isinstance(info[0]["ts"], datetime.datetime))
         self.assertTrue(isinstance(info[0]["millis"], types.FloatType))
 
+    def test_iteration(self):
+        db = self.connection.test
+
+        def iterate():
+            [a for a in db]
+
+        self.assertRaises(TypeError, iterate)
+
+    def test_errors(self):
+        db = self.connection.test
+
+        db.reset_error_history()
+        self.assertEqual(None, db.error())
+        self.assertEqual(None, db.previous_error())
+
+        db._command({"forceerror": 1})
+        self.assertTrue(db.error())
+        self.assertTrue(db.previous_error())
+
+        db._command({"forceerror": 1})
+        self.assertTrue(db.error())
+        prev_error = db.previous_error()
+        self.assertEqual(prev_error["nPrev"], 1)
+        del prev_error["nPrev"]
+        self.assertEqual(db.error(), prev_error)
+
+        db.test.find_one()
+        self.assertEqual(None, db.error())
+        self.assertTrue(db.previous_error())
+        self.assertEqual(db.previous_error()["nPrev"], 2)
+
+        db.reset_error_history()
+        self.assertEqual(None, db.error())
+        self.assertEqual(None, db.previous_error())
+
+    def test_password_digest(self):
+        db = self.connection.test
+
+        self.assertRaises(TypeError, db._password_digest, 5)
+        self.assertRaises(TypeError, db._password_digest, True)
+        self.assertRaises(TypeError, db._password_digest, None)
+
+        self.assertTrue(isinstance(db._password_digest("password"), types.UnicodeType))
+        self.assertEqual(db._password_digest("password"), u"3b483f665f97e2fdf97575e72321fc5b")
+        self.assertEqual(db._password_digest("password"), db._password_digest(u"password"))
+
+    def test_authenticate(self):
+        db = self.connection.test
+        db.system.users.remove({})
+        db.system.users.insert({"user": u"mike", "pwd": db._password_digest("password")})
+
+        self.assertRaises(TypeError, db.authenticate, 5, "password")
+        self.assertRaises(TypeError, db.authenticate, "mike", 5)
+
+        self.assertFalse(db.authenticate("mike", "not a real password"))
+        self.assertFalse(db.authenticate("faker", "password"))
+        self.assertTrue(db.authenticate("mike", "password"))
+        self.assertTrue(db.authenticate(u"mike", u"password"))
+
+        # just make sure there are no exceptions here
+        db.logout()
+        db.logout()
+
+    # TODO some of these tests belong in the collection level testing.
     def test_save_find_one(self):
         db = Database(self.connection, "test")
         db.test.remove({})
